/*
	ProWesS reader - a HTML file browser

	PROforma, PROGS Font & Raster Manager
	ProWesS, PROGS Windowing System

	FileDisplay routine - to load and display a file

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 18 November 1995
	25.11.1995 use OPEN_READ (was faulty OPEN_OLD)
			   better error handling and free stream afterwards
	03.12.1995 proper display of error message
	11.12.1995 printing handled
	06.01.1996 better error handling on DoDisplay (could crash before)
	10.01.1996 previous fix could case DoDisplay to return without an error
			   occuring (because of not initialised err).
	10.06.1996 progres report when loading (not yet when printing)
	13.06.1996 now also when printing, and proper cancel
	27.09.1996 small (5pt) margin now when printing
	12.10.1996 keeping a margin when printing now possible
	09.02.1997 now possible to limit which pages have to be printed
*/

#include "reader_h"
#include "HyperText_h"
#include "HTML_h"
#include "HTMLdtd_h"
#include "BackLog_h"

#define PASSTEXT "page %3d, pass %3d of %3d"
#define FILETEXT "reading %s"
#define CALC	 "calculating contents of page"
#define REND	 "rendering a pass"
#define SHOW	 "transfering data to printer"
#define ERRCANCEL "\n***** CANCELLED *****"
#define MARGIN short2pt(5)

/* external functions */
Error DisplayError(Error error);

/* need a wrapper function to display posible errors */
static Error DoDisplay(Global *g, char *position);
static Error PrintDisplay(Global *g);

/* actions for progres report window */
Error cancel(PWObject object)
{
	Global *g;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);
	JOBEventSend(g->reader,EVENT_LEVEL1);
	return ERR_OK;
}
static Error redraw1(PWObject object)
{
	Global *g;
	Error err;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);
	return PWChange(g->part, PW_INFOSTRING_TEXT, g->parttext,	NULL);
}
static Error redraw2(PWObject object)
{
	Global *g;
	Error err;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);
	catch( PWChange(g->file, PW_INFOSTRING_TEXT, g->filetext,	NULL) );
	catch( PWChange(g->acti, PW_INFOSTRING_TEXT, g->actitext,	NULL) );
	catch( PWChange(g->pass, PW_INFOSTRING_TEXT, g->passtext,	NULL) );
	return PWChange(g->part, PW_INFOSTRING_TEXT, g->parttext,	NULL);
}
static Error progres(Global *g)
{
	Error err;
	PWObject win;

	catch( PWCreate(NULL,&win,PW_TYPE_OUTLINE,
				PW_OUTLINE_QUIT,
				PW_OUTLINE_ACTION_QUIT, cancel,
				NULL) );
	catch( PWCreate(win,&g->file,PW_TYPE_INFOSTRING,
				PW_INFOSTRING_TEXT, g->filetext,
				PW_INFOSTRING_AUTOSIZE, FALSE,
				NULL) );
	catch( PWCreate(win,&g->part,PW_TYPE_INFOSTRING,
				PW_INFOSTRING_AUTOSIZE, FALSE,
				PW_INFOSTRING_TEXT, g->parttext,
				PW_GLOBAL_AUXILIARY, g,
				PW_SYSTEM_ACTION_EVENT1, redraw1,
				PW_SYSTEM_ACTION_EVENT2, redraw2,
				NULL) );
	if (g->printing)
	{
		catch( PWCreate(win,&g->pass,PW_TYPE_INFOSTRING,
				PW_INFOSTRING_TEXT, g->passtext,
				PW_INFOSTRING_AUTOSIZE, FALSE,
				NULL) );
		catch( PWCreate(win,&g->acti,PW_TYPE_INFOSTRING,
				PW_INFOSTRING_TEXT, g->actitext,
				PW_INFOSTRING_AUTOSIZE, FALSE,
				NULL) );
	}
	else
		g->pass=g->acti=NULL;
	return PWActivate(win);
}

Error FileDisplay(Global *g, char *position)
{
	Error err;

	g->pagenumber=1;

	STRFormatPutSafe(g->filetext,MAX_MESSAGE,FILETEXT,g->filename);
	STRCopy("loaded      0 bytes of      0 bytes",g->parttext);
	g->actitext=CALC;
	STRFormatPutSafe(g->passtext,MAX_MESSAGE,PASSTEXT,0,0,0);
	JOBMe(&g->reader);
	JOBCall(progres,"reader progress report",1,DEFAULT_STACK,
			&g->progres,1,g);

	err=DoDisplay(g,position);

	JOBZap(g->progres,ERR_OK);

	if (err) DisplayError(err);

	/* redisplay the file */
	if (!g->printing) ApplicRedisplay(g);

	return ERR_OK;
}

#define BUFFLEN 200
static Error DoDisplay3(Global *g, char *position, Channel file, Gstate gstate, pt width)
{
	Error err;
	HTStructured *HTMLStream;
	HTStream *SGMLStream;
	char buffer[BUFFLEN];
	Size got,count=0;

	/* clear the text we had (using correct parameters) */
	catch( HTReInit(g, gstate, g->look, &g->text, width) );
	STRCopy(g->directory,g->text->directory);

	/* send file through SMGL and HTML processing streams */
	catch( HTML_new(g->text,g->printing,g->printsub,&HTMLStream) );
	catch( SGML_new(&HTML_dtd,HTMLStream,&SGMLStream) );

	IOGetBytes(file, BUFFLEN, buffer, &got);
	while (got>0)
	{
		/* check for cancel */
		WindowEvent event=EVENT_LEVEL1;
		JOBEventWait(NOWAIT,&event);
		if (event)
		{
			SGMLParser.put_block(SGMLStream,ERRCANCEL,sizeof(ERRCANCEL));
			HTContentsDelete(g->text);
			break;
		}

		count+=got;
		STRFormatPutSafe(g->parttext,MAX_MESSAGE,"read %d bytes of %d bytes",
						 count,g->filelength);
		JOBEventSend(g->progres,EVENT_LEVEL1);
		err=SGMLParser.put_block(SGMLStream,buffer,got);
		if (err) break;
		IOGetBytes(file, BUFFLEN, buffer, &got);
	}
	err=SGMLParser._free(SGMLStream);

	/* add new file in history */
	BackLogAdd(g);

	/* go to correct position in file */
	ApplicPosition(g,position);

	/* done */
	return err;
}

static Error DoDisplay2(Global *g, char *position, Gstate gstate, pt width)
{
	Error err;
	Channel file;

	/* see if the new file can be opened */
	catch( IOOpenPath(g->filename,OPEN_READ,g->directory,&file) );

	/* update filename/directory -> possible to move down in directory tree */
	IOName(file,g->filename,g->directory);
	IOLength(file,&g->filelength);

	err=DoDisplay3(g,position,file,gstate,width);
	IOClose(file);
	return err;
}

static Error DoDisplay(Global *g, char *position)
{
	Error err=ERR_OK;
	Gstate gstate;
	pt width;
	char loop=TRUE;
	char filename[IO_MAXFULLNAME];
	char directory[IO_MAXDIRECTORY];

	STRCopy(g->filename,filename);
	STRCopy(g->directory,directory);
	if (!filename[0]) return ERR_OK;

	if (!g->printing)
	{
		PWQuery(g->outline,PW_SYSTEM_GSTATE,&gstate);
		width=g->canvaswidth;
	}
	else
	{
		static const Bbox FullPage={ short2pt(2000),short2pt(2000), 0,0 };
		static const Bbox MargPage={ short2pt(2000),short2pt(2000), short2pt(72),0 };
		catch( PFGstateInit((int)g->drivername,g->devicename,
			   g->printmarg?&MargPage:&FullPage,&gstate,&g->passes) );
		PFPageBboxFullGet(gstate,&g->pagebbox);
		width=g->pagebbox.xsiz-2*MARGIN;
	}

	while (loop && !err)
	{
		if (g->printing)
		{
			STRFormatPutSafe(g->filetext,MAX_MESSAGE,FILETEXT,g->filename);
			g->actitext=CALC;
			WINEventSend(g->progres,EVENT_LEVEL2);
		}

		err=DoDisplay2(g,position,gstate,width);
		if (err) break;

		if (g->printing)
			PrintDisplay(g);

		loop=FALSE;
		if (g->text->cont_curr)
		{
			if (g->text->cont_curr=g->text->cont_curr->next)
				loop=TRUE;
		}
		else
		if (g->text->cont_curr=g->text->cont_first)
			loop=TRUE;
		if (loop)
		{
			STRCopy(g->text->cont_curr->filename,g->filename);
			STRCopy(g->text->cont_curr->directory,g->directory);
		}
	}

	if (g->printing && g->text)
		PFGstateRemove(g->text->gstate);

	/* remove contents */
	if (g->text)
		HTContentsDelete(g->text);

	/* restore filename and directory - and set position for reload */
	if (g->printing)
	{
		STRCopy(filename,g->filename);
		STRCopy(directory,g->directory);
		STRCopySafe(position,g->position,IO_MAXFULLNAME);
	}

	return err;
}

static Error PrintDisplay(Global *g)
{
	pt xpos,ypos;
	int pass;
	char display;
	TextLine *startline, *line;
	Gstate gstate;
	pt yend, yfooter;
	Bbox bbox;
	int cancelled=FALSE;

	if (!g->text) return ERR_OK;

	startline=g->text->lines.next;
	gstate=g->text->gstate;
	yend=g->pagebbox.ysiz-2*g->look->bodyfontsize+g->pagebbox.yorg-2*MARGIN;

	PFPaperColourGray(gstate,0);
	PFColourGray(gstate,pt_hundred);
	PFPathStroked(gstate);

	PFFontLoad(gstate,g->look->fonts[ROMAN-1]);
	PFFontScale(gstate,g->look->bodyfontsize);
	PFFontBbox(gstate, &bbox);
	yfooter=g->pagebbox.yorg+g->pagebbox.ysiz-bbox.ysiz+bbox.yorg-MARGIN;

	while (!cancelled && startline!=&g->text->lines)
	{
		display=(g->pagenumber>=g->printfirst && g->pagenumber<=g->printlast);

		for (pass=0 ; pass<(display?g->passes:1) ; pass++)
		{
			WindowEvent event=EVENT_LEVEL1;
			JOBEventWait(NOWAIT,&event);
			if (event)
			{
				cancelled=TRUE;
				HTContentsDelete(g->text);
			}

			g->actitext=REND;
			STRFormatPutSafe(g->passtext,MAX_MESSAGE,PASSTEXT,
									g->pagenumber,pass+1,g->passes);
			WINEventSend(g->progres,EVENT_LEVEL2);
			line=startline;
			xpos=g->pagebbox.xorg+MARGIN;
			ypos=g->pagebbox.yorg+MARGIN;

			if (display)
			{
				PFPageClear(gstate);

				if (!cancelled)
				{
					/* select current font and size */
					PFFontLoad(gstate, line->styleline->font);
					PFFontScale(gstate, line->styleline->size);

					/* draw all the lines */
					for ( ; line!=&g->text->lines && ypos+line->height<yend ;
							ypos+=line->height, line=line->next )
					{
						LineElement *el;

						/* move to start of line */
						PFMoveTo(gstate, xpos, ypos+line->baseline);

						/* run through elements in line and draw all */
						for (el=line->elements ; el ; el=el->next)
							el->Draw(gstate,el);
					}

					/* now draw the footer */
					{
						char number[10];
						pt width;

						PFFontLoad(gstate,g->look->fonts[ROMAN-1]);
						PFFontScale(gstate,g->look->bodyfontsize);
						PFMoveTo(gstate,xpos,yfooter);
						PFStringShow(gstate,g->text->title);
						STRFormatPut(number,"%d",g->pagenumber);
						PFStringWidth(gstate,number,&width);
						PFMoveTo(gstate,xpos+g->pagebbox.xsiz-width-2*MARGIN,yfooter);
						PFStringShow(gstate,number);
					}
				}

				/* draw this pass */
				g->actitext=SHOW;
				WINEventSend(g->progres,EVENT_LEVEL2);
				PFPageShow(gstate);
			}
			else
			{
				/* make sure we move to the end of the page */
				for ( ; line!=&g->text->lines && ypos+line->height<yend ;
						ypos+=line->height, line=line->next );
			}
		}
		g->pagenumber++;
		if (g->pagenumber>g->printlast)
		{
			cancelled=TRUE;
			HTContentsDelete(g->text);
		}
		startline=line;
	}
	return ERR_OK;
}
