/*
		CHUNK HANDLING: FLEXIBLE ARRAYS
		(c) COPYRIGHT MIT 1995.
		Please first read the full copyright statement in the file COPYRIGHT.

	adjusted for use in ProWesS reader
	by Joachim Van der Auwera
	for PROGS, Professional & Graphical Software
	started 14 November 1995
*/

#include "mem_h"

#include "HTChunk_h"

#define catch(x) if (err=(x)) return err; else

/*	Create a chunk with a certain allocation unit */
/*	--------------------------------------------- */

Error HTChunkCreate(Size grow, HTChunk **ret)
{
	Error err;

	catch( MEMAllocate(sizeof(HTChunk),ret) );

	(*ret)->growby = grow;
	return err;
}


/*	Clear a chunk of all data */
/*	------------------------- */

Error HTChunkClear(HTChunk *ch)
{
	if (ch->data)
	{
		MEMRelease(ch->data);
		ch->data = NULL;
	}
	ch->size = 0;
	ch->allocated = 0;
	return ERR_OK;
}


/*	Free a chunk */
/*	------------ */

Error HTChunkFree(HTChunk *ch)
{
	if (ch->data) MEMRelease(ch->data);
	return MEMRelease(ch);
}


/*	Append a character */
/*	------------------ */

Error HTChunkPutc(HTChunk *ch, char c)
{
	Error err=ERR_OK;

	if (ch->size >= ch->allocated-1)
	{
		if (ch->data)
		{
			char *new;
			catch( MEMAllocate(ch->allocated+ch->growby,&new) );
			MEMCopy(ch->allocated,ch->data,new);
			MEMRelease(ch->data);
			ch->data=new;
		}
		else
		{
			catch( MEMAllocate(ch->growby,&ch->data) );
		}
		ch->allocated+=ch->growby;
	}
	ch->data[ch->size++] = c;
	return err;
}


/*	Ensure a certain size */
/*	--------------------- */

Error HTChunkEnsure(HTChunk *ch, Size needed)
{
	Error err=ERR_OK;
	char *new;

	if (needed <= ch->allocated) return err;
	needed = ( ((needed-1)/ch->growby) +1) * ch->growby;

	catch( MEMAllocate(needed,&new) );

	if (ch->data)
	{
		MEMCopy(needed,ch->data,new);
		MEMRelease(ch->data);
	}
	ch->data=new;
	ch->allocated=needed;

	return err;
}


/*	Terminate a chunk */
/*	----------------- */

Error HTChunkTerminate(HTChunk *ch)
{
	return HTChunkPutc(ch,'\0');
}


/*	Append a string */
/*	--------------- */

Error HTChunkPuts(HTChunk *ch, const char *s)
{
	Error err;
	for (; *s; s++)
		catch( HTChunkPutc(ch, *s) );
	return err;
}
