/*
		STATIC DTD FOR HTML
		(c) COPYRIGHT MIT 1995.
		Please first read the full copyright statement in the file COPYRIGH.

	adjusted for use in ProWesS reader
	by Joachim Van der Auwera
	for PROGS, Professional & Graphical Software
	started 16 November 1995
	28.11.1995 added reg and copy entities (which also exist in Netscape)
	11.12.1995 reduced to minimal version
*/

/* Library include files */
#include "HTMLdtd_h"

/*	Entity Names
**	------------
**
**	This table must be matched exactly with ALL the translation tables
*/

static const char *entities[HTML_ENTITIES] = {
  "AElig",	/* capital AE diphthong (ligature) */ 
  "Aacute", /* capital A, acute accent */ 
  "Acirc",	/* capital A, circumflex accent */ 
  "Agrave", /* capital A, grave accent */ 
  "Aring",	/* capital A, ring */ 
  "Atilde", /* capital A, tilde */ 
  "Auml",	/* capital A, dieresis or umlaut mark */ 
  "Ccedil", /* capital C, cedilla */ 
  "ETH",	/* capital Eth, Icelandic */ 
  "Eacute", /* capital E, acute accent */ 
  "Ecirc",	/* capital E, circumflex accent */ 
  "Egrave", /* capital E, grave accent */ 
  "Euml",	/* capital E, dieresis or umlaut mark */ 
  "Iacute", /* capital I, acute accent */ 
  "Icirc",	/* capital I, circumflex accent */ 
  "Igrave", /* capital I, grave accent */ 
  "Iuml",	/* capital I, dieresis or umlaut mark */ 
  "Ntilde", /* capital N, tilde */ 
  "Oacute", /* capital O, acute accent */ 
  "Ocirc",	/* capital O, circumflex accent */ 
  "Ograve", /* capital O, grave accent */ 
  "Oslash", /* capital O, slash */ 
  "Otilde", /* capital O, tilde */ 
  "Ouml",	/* capital O, dieresis or umlaut mark */ 
  "THORN",	/* capital THORN, Icelandic */ 
  "Uacute", /* capital U, acute accent */ 
  "Ucirc",	/* capital U, circumflex accent */ 
  "Ugrave", /* capital U, grave accent */ 
  "Uuml",	/* capital U, dieresis or umlaut mark */ 
  "Yacute", /* capital Y, acute accent */ 
  "aacute", /* small a, acute accent */
  "acirc",	/* small a, circumflex accent */
  "acute",	/* acute accent */
  "aelig",	/* small ae diphthong (ligature) */ 
  "agrave", /* small a, grave accent */ 
  "amp",	/* ampersand */ 
  "aring",	/* small a, ring */ 
  "atilde", /* small a, tilde */ 
  "auml",	/* small a, dieresis or umlaut mark */
  "brvbar", /* broken vertical bar */
  "ccedil", /* small c, cedilla */
  "cedil",	/* cedilla */
  "cent",	/* sent sign */
  "copy",	/* copyright sign */
  "curren", /* currency sign */
  "deg",	/* degree sign */
  "divide", /* divide sign */
  "eacute", /* small e, acute accent */ 
  "ecirc",	/* small e, circumflex accent */ 
  "egrave", /* small e, grave accent */ 
  "eth",	/* small eth, Icelandic */ 
  "euml",	/* small e, dieresis or umlaut mark */
  "frac12", /* fraction 1/2 */
  "frac14", /* fraction 1/4 */
  "frac34", /* fraction 3/4 */
  "gt", 	/* greater than */ 
  "iacute", /* small i, acute accent */ 
  "icirc",	/* small i, circumflex accent */
  "iexcl",	/* inverted exclamation mark */
  "igrave", /* small i, grave accent */
  "iquest", /* inverted question mark */
  "iuml",	/* small i, dieresis or umlaut mark */
  "laquo",	/* angle quotation mark, left */
  "lt", 	/* less than */
  "macr",	/* macron */
  "micro",	/* micro sign */
  "middot", /* middle dot */
  "nbsp",	/* (no break) space */
  "not",	/* not sign */
  "ntilde", /* small n, tilde */ 
  "oacute", /* small o, acute accent */ 
  "ocirc",	/* small o, circumflex accent */ 
  "ograve", /* small o, grave accent */
  "ordf",	/* ordinal indicator, feminine */
  "ordm",	/* ordinal indicator, masculine */
  "oslash", /* small o, slash */ 
  "otilde", /* small o, tilde */ 
  "ouml",	/* small o, dieresis or umlaut mark */
  "para",	/* paragraph sign (pilcrow) */
  "plusmn", /* plus-or-minus sign */
  "pound",	/* pound sign */
  "quot",	/* double quote sign  - June 1994 */
  "raquo",	/* angle quotation mark, right */
  "reg",	/* trademark (as in Netscape) - PROGS */
  "sect",	/* section sign */
  "shy",	/* soft hyphen */
  "sup1",	/* superscript 1 */
  "sup2",	/* superscript 2 */
  "sup3",	/* superscript 3 */
  "szlig",	/* small sharp s, German (sz ligature) */
  "thorn",	/* small thorn, Icelandic */
  "times",	/* multiply sign */
  "uacute", /* small u, acute accent */ 
  "ucirc",	/* small u, circumflex accent */ 
  "ugrave", /* small u, grave accent */
  "uml",	/* umlaut or dieresis */
  "uuml",	/* small u, dieresis or umlaut mark */ 
  "yacute", /* small y, acute accent */
  "yen",	/* yen sign */
  "yuml",	/* small y, dieresis or umlaut mark */ 
};



/*		Attribute Lists
**		---------------
**
**	Lists must be in alphatbetical order by attribute name
**	The tag elements contain the number of attributes
*/
static attr no_attr[1] = 
	{{ 0 }};

static attr a_attr[HTML_A_ATTRIBUTES+1] = {     /* Anchor attributes */
	{ "HREF"},
	{ "ID" },
	{ "NAME" },                     /* Should be ID */
	{ "PRINT" },
	{ "REL" },                      /* Relationship */
	{ "REV" },                      /* Reverse relationship */
	{ 0 }                           /* Terminate list */
};

static attr img_attr[HTML_IMG_ATTRIBUTES+1] = { /* IMG attributes */
	{ "ALT" },
	{ "HEIGHT" },
	{ "SRC" },
	{ "UNITS" },
	{ "WIDTH" },
	{ 0 }                           /* Terminate list */
};

/*	Elements
**	--------
**
**	Must match definitions in HTMLdtd.h !
**	Must be in alphabetical order.
**
**	HTML is included to allow HTML documents to be parsed as a subset.
**	TITLE is changed to type SGML_MIXED so it get parsed as well
**	Henrik 08/03-94
**
**	  Name, 	Attributes, 		content
*/
static HTTag tags[HTML_ELEMENTS] = {
	{ "A"           , a_attr,       HTML_A_ATTRIBUTES,      SGML_MIXED },
	{ "ADDRESS"     , no_attr,      0,                      SGML_MIXED },
	{ "B"           , no_attr,      0,                      SGML_MIXED },
	{ "BLOCKQUOTE"  , no_attr,      0,                      SGML_MIXED },
	{ "BQ"          , no_attr,      0,                      SGML_MIXED },
	{ "BR"          , no_attr,      0,                      SGML_EMPTY },
	{ "CITE"        , no_attr,      0,                      SGML_MIXED },
	{ "CODE"        , no_attr,      0,                      SGML_MIXED },
	{ "COMMENT"     , no_attr,      0,                      SGML_MIXED },
	{ "DD"          , no_attr,      0,                      SGML_EMPTY },
	{ "DIR"         , no_attr,      0,                      SGML_MIXED },
	{ "DL"          , no_attr,      0,                      SGML_MIXED },
	{ "DT"          , no_attr,      0,                      SGML_EMPTY },
	{ "EM"          , no_attr,      0,                      SGML_MIXED },
	{ "H1"          , no_attr,      0,                      SGML_MIXED },
	{ "H2"          , no_attr,      0,                      SGML_MIXED },
	{ "H3"          , no_attr,      0,                      SGML_MIXED },
	{ "H4"          , no_attr,      0,                      SGML_MIXED },
	{ "H5"          , no_attr,      0,                      SGML_MIXED },
	{ "H6"          , no_attr,      0,                      SGML_MIXED },
	{ "H7"          , no_attr,      0,                      SGML_MIXED },
	{ "HR"          , no_attr,      0,                      SGML_EMPTY },
	{ "I"           , no_attr,      0,                      SGML_MIXED },
	{ "IMG"         , img_attr,     HTML_IMG_ATTRIBUTES,    SGML_EMPTY },
	{ "KBD"         , no_attr,      0,                      SGML_MIXED },
	{ "LH"          , no_attr,      0,                      SGML_MIXED },
	{ "LI"          , no_attr,      0,                      SGML_EMPTY },
	{ "LISTING"     , no_attr,      0,                      SGML_LITERAL },
	{ "MENU"        , no_attr,      0,                      SGML_MIXED },
	{ "OL"          , no_attr,      0,                      SGML_MIXED },
	{ "P"           , no_attr,      0,                      SGML_EMPTY },
	{ "PLAINTEXT"   , no_attr,      0,                      SGML_LITERAL },
	{ "PRE"         , no_attr,      0,                      SGML_MIXED },
	{ "SAMP"        , no_attr,      0,                      SGML_MIXED },
	{ "STRONG"      , no_attr,      0,                      SGML_MIXED },
	{ "TITLE"       , no_attr,      0,                      SGML_MIXED },
	{ "TT"          , no_attr,      0,                      SGML_MIXED },
	{ "UL"          , no_attr,      0,                      SGML_MIXED },
	{ "VAR"         , no_attr,      0,                      SGML_MIXED },
	{ "XMP"         , no_attr,      0,                      SGML_LITERAL }
};


const SGML_dtd HTML_dtd = {
	tags,
	HTML_ELEMENTS,
	entities,
	sizeof(entities)/sizeof(char**)
};
