#ifndef _READER_HTSTRUCT_H
#define _READER_HTSTRUCT_H
/*
		STRUCTURED STREAM DEFINITION
		(c) COPYRIGHT MIT 1995.
		Please first read the full copyright statement in the file COPYRIGH.

	This module is a part of the W3C Reference Library.
   
	adjusted for use in ProWesS reader
	by Joachim Van der Auwera
	for PROGS, Professional & Graphical Software
	started 15 November 1995
*/

#include "HTStream_h"

/*
	A structured object is something which can reasonably be represented in
	for example SGML.  I'll rephrase that.  A structured object is an ordered
	tree-structured arrangement of data which is representable as text. An
	example is the SGML parser which outputs to a Structured Object. A
	Structured object can output its contents to another Structured Object.
	It's a kind of typed stream. The architecure is largely Dan Conolly's.
	Elements and entities are passed to the sob by number, implying a knowledge
	of the DTD.
   
	The Structured Stream is a subclass of a Generic Stream Object. As always,
	we don't have classes in basic C so we have to do this by hand !
   
	NOTE: The put_block method was write, but this upset systems which had
	macros for write().

	Joachim : The syslib Error type is used for return code.
*/

typedef struct _HTStructured HTStructured;

typedef struct _HTStructuredClass {

		/* This first bit is inherited from HTStreamClass. */
		/* But now with a HTStructured as parameter. */

	char *name;
	Error (*flush)(HTStructured * me);
	Error (*_free)(HTStructured * me);
	Error (*abort)(HTStructured * me, Error e);
	Error (*put_character)(HTStructured *me, char ch);
	Error (*put_string)(HTStructured *me, const char *str);
	Error (*put_block)(HTStructured *me, const char *str, Size length);

		/* The specific stuff. */

		/* start a SGML element, including parameters, called after */
		/* processing a recognized <...> start tag */

	Error (*start_element)(HTStructured *me,
			int 			element_number,
			const char *	attribute_present,
			const char **	attribute_value);

		/* end an SGML element, called after a </...> end tag */

	Error (*end_element)(HTStructured *  me,
			int 			element_number);

		/* add entity to structured stream, e.g. "&lt;" */

	Error (*put_entity)(HTStructured *me,
			int 			entity_number);
				
} HTStructuredClass;

#endif _READER_HTSTRUCT_H
