/*
	ProWesS reader - a HTML file browser

	PROforma, PROGS Font & Raster Manager
	ProWesS, PROGS Windowing System

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 14 November 1995
*/

#include "io_h"

#include "SGML_h"
#include "HTMLdtd_h"

#define catch(x) if (err=(x)) return err; else

typedef struct _HTStructured {
	HTStructuredClass *isa;
	Channel out;
};

Error TestFlush(HTStructured * me)
{
	return IOPutString(me->out,"[flush]",NULL);
}

Error TestFree(HTStructured * me)
{
	return IOPutString(me->out,"[_free]",NULL);
}

Error TestAbort(HTStructured * me, Error e)
{
	return IOPutString(me->out,"[abort]",NULL);
}

Error TestPutc(HTStructured *me, char ch)
{
	return IOPut(me->out,ch);
}

Error TestPuts(HTStructured *me, const char *str)
{
	while (*str)
		TestPutc(me,*str++);
	return ERR_OK;
}

Error TestPutb(HTStructured *me, const char *str, Size length)
{
	while (length--)
		TestPutc(me,*str++);
	return ERR_OK;
}

Error TestElementStart(HTStructured *me, int element_number,
			const char *	attribute_present,
			const char **	attribute_value)
{
	char line[80];
	STRFormatPut(line,"[start elem %d]",element_number);
	return IOPutString(me->out,line,NULL);
}

Error TestElementEnd(HTStructured *me, int element_number)
{
	char line[80];
	STRFormatPut(line,"[end elem %d]",element_number);
	return IOPutString(me->out,line,NULL);
}

Error TestEntityPut(HTStructured *me, int entity_number)
{
	char line[80];
	STRFormatPut(line,"[entity %d]",entity_number);
	return IOPutString(me->out,line,NULL);
}


HTStructuredClass SGMLTestClass={
	"SGML Parser Test Class",
	TestFlush,
	TestFree,
	TestAbort,
	TestPutc,
	TestPuts,
	TestPutb,
	TestElementStart,
	TestElementEnd,
	TestEntityPut
};

struct _HTStream { HTStreamClass *isa; };

Error init()
{
	Error err;
	char ch;
	Channel in,out;
	HTStream *Stream;
	HTStructured SGMLtest;

	catch( IOOpen("win1_html_tables.txt",OPEN_READ,&in) );
	catch( IOOpen("ram1_smgltest_out"	,OPEN_OVER,&out) );

	SGMLtest.isa=&SGMLTestClass;
	SGMLtest.out=out;
	catch( SGML_new(&HTML_dtd, &SGMLtest, &Stream) );

	while (!IOGet(in,&ch))
		Stream->isa->put_character(Stream,ch);

	return err;
}
