/*
	ProWesS reader - a HTML file browser

	PROforma, PROGS Font & Raster Manager
	ProWesS, PROGS Windowing System

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 16 November 1995
	03.12.1995 back action routine added
	06.01.1995 -help -file -pos -dir commmandline parameters added
	10.04.1996 ProWesS default fontsize now used when bodyfontsize configured
			   to zero (which is the default)
	30.08.1996 CurrentDirectory can now be configured (as Roy Wood asked)
	12.10.1996 keeping a margin when printing now possible
	09.02.1997 allow the range of page to be printed to be chosen
*/

#include <mem.h>
#include <str.h>
#include "reader.h"
#include "HyperText.h"
#include "BackLog.h"
#include "Bookmark.h"

#define FONTSIZE_MAXLEN 20
#define PAGENO_MAXLEN 10

/* configurable aspects of menu */

extern char ItemBack[];
extern char ItemLoad[];
extern char ItemReload[];
extern char ItemPrint[];
extern char ItemStyles[];
extern char ItemBookmarks[];
extern char ItemHistory[];
extern char ItemBookmarkAdd[];
extern char ItemBookmarkLoad[];
extern char ItemBookmarkSave[];
extern char ItemPrintSub[];
extern char ItemPrintMargin[];
extern char LabelFrom[], LabelTo[];
extern char LabelTitle[];
extern char LabelFontSize[];
extern char LabelFontRoman[];
extern char LabelFontItalic[];
extern char LabelFontBold[];
extern char LabelFontBoldItalic[];
extern char LabelFontTypeWriter[];
extern char LabelDevice[];
extern char LabelDriver[];
extern char DefaultDriver[];
extern char DefaultDevice[];
extern char ManualDirectory[];
extern char CurrentDirectory[];
extern short WindowSizeX, WindowSizeY;

/* external routines */
Error DriverSelect(char *menu_name, char screen, char def, char *drivername);
Error FontSelect(char *menu_name, char *fontname);

/* forward declarations */
static Error fsel(PWObject object);
static Error reload(PWObject object);
static Error back(PWObject object);
static Error history(PWObject object);
static Error bookmark(PWObject object);
static Error style(PWObject object);
static Error print(PWObject object);
static Error loadfile(PWObject object, char *directory, char *filename);

Error init(char *param, short chcount, Channel *channels)
{
	Global g;
	Error err;
	PWObject box;
	char **params;
	char help=FALSE;

	LinkPROforma(); 				/* make sure PROforma linkage exists */
	MEMClear(sizeof(Global),&g);
	g.printsub=TRUE;
	g.printfirst=1;
	g.printlast=999;
	STRCopy(DefaultDriver,g.drivername);
	STRCopy(DefaultDevice,g.devicename);

	catch( StyleCreate(&g.look) );
	catch( BackLogInit(&g) );
	catch( BookmarkInit(&g) );

	/* handle the command line parameters */
	if (!STRBreak(param,&params))
	{
		char **orgparams=params;

		for ( ; *params ; params++)
		{
			if (STRSameCI("-help",*params))
				help=TRUE;
			else
			if (STRSameCI("-file",*params) && *(params+1))
			{
				params++;
				STRCopySafe(*params,g.filename,IO_MAXFULLNAME);
			}
			else
			if (STRSameCI("-pos",*params) && *(params+1))
			{
				params++;
				STRCopySafe(*params,g.position,IO_MAXFULLNAME);
			}
			else
			if (STRSameCI("-dir",*params) && *(params+1))
			{
				params++;
				STRCopySafe(*params,g.directory,IO_MAXDIRECTORY);
			}
		}
		MEMRelease(orgparams);
	}

	catch( PWCreate(NULL,&g.outline,PW_TYPE_OUTLINE,
					PW_OUTLINE_SLEEP,
					PW_OUTLINE_QUIT,
					PW_OUTLINE_QUIT_KEYPRESS, 27,
					PW_OUTLINE_QUIT_CONFIRM, TRUE,
					PW_WINDOW_SIZE, short2pt(WindowSizeX), short2pt(WindowSizeY),
					PW_GLOBAL_AUXILIARY, &g,
					PW_SYSTEM_ACTION_INIT, reload,
					PW_OUTLINE_HELP,
					PW_OUTLINE_HELP_DIRECTORY, ManualDirectory,
					PW_OUTLINE_HELP_FILE, "reader_html",
					NULL) );

	if (!g.look->bodyfontsize)
		PWQuery(g.outline,PW_DEFAULT_FONTSIZE,&g.look->bodyfontsize);

	catch( PWCreate(g.outline,&g.fsel,PW_TYPE_FILESELECT,
					PW_FILESELECT_EXTENSION, "html;htm",
					PW_FILESELECT_DIRECTORY, CurrentDirectory,
					PW_FILESELECT_ACTION, loadfile,
					NULL) );

	catch( PWCreate(g.outline,&box,PW_TYPE_DIRECTION,
					PW_SCALE_FACTOR, 0,
					NULL) );

	catch( PWCreate(box,NULL,PW_TYPE_LOOSE_ITEM,
					PW_LOOSE_TEXT, ItemBack,
					PW_LOOSE_CHANGE_STATUS, FALSE,
					PW_LOOSE_ACTION_HIT, back,
					PW_LOOSE_KEYPRESS, 'b',
					NULL) );
	if (!help)
		catch( PWCreate(box,NULL,PW_TYPE_LOOSE_ITEM,
					PW_LOOSE_TEXT, ItemLoad,
					PW_LOOSE_CHANGE_STATUS, FALSE,
					PW_LOOSE_ACTION_HIT, fsel,
					PW_LOOSE_KEYPRESS, 'l',
					NULL) );
	catch( PWCreate(box,NULL,PW_TYPE_LOOSE_ITEM,
					PW_LOOSE_TEXT, ItemReload,
					PW_LOOSE_CHANGE_STATUS, FALSE,
					PW_LOOSE_ACTION_HIT, reload,
					PW_LOOSE_KEYPRESS, 'r',
					NULL) );
	catch( PWCreate(box,NULL,PW_TYPE_LOOSE_ITEM,
					PW_LOOSE_TEXT, ItemPrint,
					PW_LOOSE_CHANGE_STATUS, FALSE,
					PW_LOOSE_ACTION_HIT, print,
					PW_LOOSE_KEYPRESS, 'p',
					NULL) );
	catch( PWCreate(box,NULL,PW_TYPE_LOOSE_ITEM,
					PW_LOOSE_TEXT, ItemStyles,
					PW_LOOSE_CHANGE_STATUS, FALSE,
					PW_LOOSE_ACTION_HIT, style,
					PW_LOOSE_KEYPRESS, 's',
					NULL) );
	if (!help)
		catch( PWCreate(box,NULL,PW_TYPE_LOOSE_ITEM,
					PW_LOOSE_TEXT, ItemBookmarks,
					PW_LOOSE_CHANGE_STATUS, FALSE,
					PW_LOOSE_ACTION_HIT, bookmark,
					PW_LOOSE_KEYPRESS, 'o',
					NULL) );
	catch( PWCreate(box,NULL,PW_TYPE_LOOSE_ITEM,
					PW_LOOSE_TEXT, ItemHistory,
					PW_LOOSE_CHANGE_STATUS, FALSE,
					PW_LOOSE_ACTION_HIT, history,
					PW_LOOSE_KEYPRESS, 'h',
					NULL) );

	catch( PWCreate(g.outline,&box,PW_TYPE_DIRECTION,
					PW_SCALE_FACTOR, 0,
					NULL) );

	catch( PWCreate(box,NULL,PW_TYPE_LABEL,
					PW_LABEL_TEXT, LabelTitle,
					PW_SCALE_FACTOR, 0,
					NULL) );

	catch( PWCreate(box,&g.title,PW_TYPE_INFOSTRING,
					PW_INFOSTRING_TEXT, " ",
					PW_INFOSTRING_AUTOSIZE, FALSE,
					NULL) );

	catch( PWCreate(g.outline,NULL,PW_TYPE_SEPARATOR,NULL) );

	catch( ApplicCreate(&g) );

	return PWActivate(g.outline);
}

/* event handlers */

static Error fsel(PWObject object)
{
	Global *g;
	Error err;

	catch( PWQuery(object,PW_GLOBAL_AUXILIARY,&g) );

	catch( PWChange(g->fsel, PW_FILESELECT_ACTIVATE, NULL) );

	/* now I really have to load the file (from the top) */
	if (g->loaded)
	{
		err=FileDisplay(g, "");
		g->loaded=FALSE;
		if (g->text) PWChange(g->title,PW_INFOSTRING_TEXT,g->text->title,NULL);
	}
	return err;
}

static Error reload(PWObject object)
{
	Global *g;
	Error err;
	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);
	PWChange(object,PW_SYSTEM_ACTION_INIT,NULL,NULL);
	err=FileDisplay(g, g->position);
	if (g->text)
		PWChange(g->title,PW_INFOSTRING_TEXT,g->text->title,NULL);
	g->loaded=FALSE;
	return err;
}

static Error loadfile(PWObject object, char *directory, char *filename)
{
	Global *g;
	Error err;

	catch( PWQuery(object,PW_GLOBAL_AUXILIARY,&g) );

	g->loaded=TRUE;
	STRCopySafe(directory,g->directory,IO_MAXDIRECTORY);
	STRCopySafe(filename, g->filename, IO_MAXNAME);

	return err;
}

static Error back(PWObject object)
{
	Global *g;
	Error err;

	catch( PWQuery(object,PW_GLOBAL_AUXILIARY,&g) );

	/* load the previous file as stored in the History */
	if (!BackLogGet(g))
	{
		err=FileDisplay(g, "");
		if (g->text) PWChange(g->title,PW_INFOSTRING_TEXT,g->text->title,NULL);
	}
	return err;
}

static Error history(PWObject object)
{
	Error err;
	Global *g;
	PWObject win=NULL,menu;

	catch( PWQuery(object,PW_GLOBAL_AUXILIARY,&g) );

	err=PWCreate(NULL,&win,PW_TYPE_OUTLINE,
					PW_OUTLINE_QUIT,
					PW_OUTLINE_QUIT_KEYPRESS, 27,
					PW_OUTLINE_TITLE_TEXT, ItemHistory,
					NULL);
	if (!err) err=PWCreate(win,&menu,PW_TYPE_MENU,
					PW_MENU_ACTION_SELECT, BackLogSelect,
					PW_MENU_VISIBLE_LINES, 4,
					NULL);
	if (!err) err=BackLogMenu(g,menu);
	if (!err) err=PWActivate(win);
	if (g->loaded)
	{
		FileDisplay(g, "");
		if (g->text) PWChange(g->title,PW_INFOSTRING_TEXT,g->text->title,NULL);
		g->loaded=FALSE;
	}
	if (win) PWRemove(win);
	return ERR_OK;
}

static Error fontsel(PWObject object)
{
	Error err;
	Global *g;
	char *font;

	catch( PWQuery(object,PW_GLOBAL_AUXILIARY,&g) );

	catch( PWQuery(object,PW_LOOSE_TEXT,&font) );
	catch( FontSelect("",font) );
	err=PWChange(object,PW_LOOSE_TEXT,font,NULL);
	return ERR_OK;
}

static Error fontsizesel(PWObject object)
{
	Error err;
	Global *g;
	char str_fontsize[FONTSIZE_MAXLEN];
	double db_fontsize;

	catch( PWQuery(object,PW_GLOBAL_AUXILIARY,&g) );
	catch( PWChange(object,PW_EDLINE_GET,FONTSIZE_MAXLEN,str_fontsize,NULL) );
	STRFormatGet(str_fontsize,NULL,"%f",&db_fontsize);
	if (db_fontsize<3.0) db_fontsize=3.0;
	if (db_fontsize>100.0) db_fontsize=100.0;
	g->look->bodyfontsize=double2pt(db_fontsize);
	STRFormatPutSafe(str_fontsize,FONTSIZE_MAXLEN,"%g",pt2double(g->look->bodyfontsize));
	err=PWChange(object,PW_EDLINE_SET,str_fontsize,NULL);
	return err;
}

static Error stylesmenuhandle(Global *g, PWObject win)
{
	Error err;
	char str_fontsize[FONTSIZE_MAXLEN];
	PWObject box;

	STRFormatPutSafe(str_fontsize,FONTSIZE_MAXLEN,"%g",pt2double(g->look->bodyfontsize));

	catch( PWCreate(win,&box,PW_TYPE_DIRECTION,NULL) );
	catch( PWCreate(box,NULL,PW_TYPE_LABEL,
						PW_SCALE_FACTOR, 0,
						PW_LABEL_TEXT, LabelFontSize,
						NULL) );
	catch( PWCreate(box,NULL,PW_TYPE_EDLINE,
						PW_EDLINE_MAXLENGTH, FONTSIZE_MAXLEN,
						PW_EDLINE_SET, str_fontsize,
						PW_EDLINE_ACTION_AFTER, fontsizesel,
						NULL) );

	catch( PWCreate(win,&box,PW_TYPE_DIRECTION,NULL) );
	catch( PWCreate(box,NULL,PW_TYPE_LABEL,
						PW_SCALE_FACTOR, 0,
						PW_LABEL_TEXT, LabelFontRoman,
						NULL) );
	catch( PWCreate(box,NULL,PW_TYPE_LOOSE_ITEM,
						PW_LOOSE_TEXT, g->look->fonts[ROMAN-1],
						PW_LOOSE_CHANGE_STATUS, FALSE,
						PW_LOOSE_AUTOSIZE, FALSE,
						PW_LOOSE_ACTION_HIT, fontsel,
						NULL) );

	catch( PWCreate(win,&box,PW_TYPE_DIRECTION,NULL) );
	catch( PWCreate(box,NULL,PW_TYPE_LABEL,
						PW_SCALE_FACTOR, 0,
						PW_LABEL_TEXT, LabelFontItalic,
						NULL) );
	catch( PWCreate(box,NULL,PW_TYPE_LOOSE_ITEM,
						PW_LOOSE_TEXT, g->look->fonts[ITALIC-1],
						PW_LOOSE_CHANGE_STATUS, FALSE,
						PW_LOOSE_AUTOSIZE, FALSE,
						PW_LOOSE_ACTION_HIT, fontsel,
						NULL) );

	catch( PWCreate(win,&box,PW_TYPE_DIRECTION,NULL) );
	catch( PWCreate(box,NULL,PW_TYPE_LABEL,
						PW_SCALE_FACTOR, 0,
						PW_LABEL_TEXT, LabelFontBold,
						NULL) );
	catch( PWCreate(box,NULL,PW_TYPE_LOOSE_ITEM,
						PW_LOOSE_TEXT, g->look->fonts[BOLD-1],
						PW_LOOSE_CHANGE_STATUS, FALSE,
						PW_LOOSE_AUTOSIZE, FALSE,
						PW_LOOSE_ACTION_HIT, fontsel,
						NULL) );

	catch( PWCreate(win,&box,PW_TYPE_DIRECTION,NULL) );
	catch( PWCreate(box,NULL,PW_TYPE_LABEL,
						PW_SCALE_FACTOR, 0,
						PW_LABEL_TEXT, LabelFontBoldItalic,
						NULL) );
	catch( PWCreate(box,NULL,PW_TYPE_LOOSE_ITEM,
						PW_LOOSE_TEXT, g->look->fonts[BOLDITALIC-1],
						PW_LOOSE_CHANGE_STATUS, FALSE,
						PW_LOOSE_AUTOSIZE, FALSE,
						PW_LOOSE_ACTION_HIT, fontsel,
						NULL) );

	catch( PWCreate(win,&box,PW_TYPE_DIRECTION,NULL) );
	catch( PWCreate(box,NULL,PW_TYPE_LABEL,
						PW_SCALE_FACTOR, 0,
						PW_LABEL_TEXT, LabelFontTypeWriter,
						NULL) );
	catch( PWCreate(box,NULL,PW_TYPE_LOOSE_ITEM,
						PW_LOOSE_TEXT, g->look->fonts[TYPEWRITER-1],
						PW_LOOSE_CHANGE_STATUS, FALSE,
						PW_LOOSE_AUTOSIZE, FALSE,
						PW_LOOSE_ACTION_HIT, fontsel,
						NULL) );

	err=PWActivate(win);
	return err;
}

static Error style(PWObject object)
{
	Error err;
	Global *g;
	PWObject win;

	catch( PWQuery(object,PW_GLOBAL_AUXILIARY,&g) );

	err=PWCreate(NULL,&win,PW_TYPE_OUTLINE,
					PW_OUTLINE_QUIT,
					PW_OUTLINE_QUIT_KEYPRESS, 27,
					PW_OUTLINE_TITLE_TEXT, ItemStyles,
					PW_GLOBAL_AUXILIARY, g,
					PW_WINDOW_NOYSCALE,
					NULL);
	if (!err)
	{
		stylesmenuhandle(g,win);
		PWRemove(win);
		if (g->text) reload(object);
	}
	return ERR_OK;
}

Error driversel(PWObject object)
{
	Global *g;
	Error err;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);

	catch(DriverSelect(NULL, FALSE, FALSE, g->drivername));

	err=PWChange(g->driver,PW_LOOSE_TEXT,g->drivername,NULL);
	return err;
}

Error devicesel(PWObject object)
{
	Global *g;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);
	PWChange(object,PW_EDLINE_GET,IO_MAXFULLNAME,g->devicename,NULL);
	return ERR_OK;
}

static Error hitprtsub(PWObject object)
{
	Global *g;
	int status;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);
	PWQuery(object,PW_LOOSE_STATUS,&status);
	g->printsub = (status==PW_STATUS_SELECTED) ? TRUE : FALSE;
	return ERR_OK;
}

static Error hitprtmarg(PWObject object)
{
	Global *g;
	int status;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);
	PWQuery(object,PW_LOOSE_STATUS,&status);
	g->printmarg = (status==PW_STATUS_SELECTED) ? TRUE : FALSE;
	return ERR_OK;
}

static Error intsel(PWObject object)
{
	Error err;
	int *value;
	Global *g;
	char string[PAGENO_MAXLEN];

	catch( PWQuery(object,PW_GLOBAL_AUXILIARY,&g) );
	catch( PWQuery(object,PW_OBJECT_AUXILIARY,&value) );
	catch( PWChange(object,PW_EDLINE_GET,PAGENO_MAXLEN,string,NULL) );
	STRFormatGet(string,NULL,"%d",value);
	return err;
}

static Error printmenuhandle(Global *g, PWObject win)
{
	Error err;
	char dummy[max(FONTSIZE_MAXLEN,PAGENO_MAXLEN)];
	PWObject box,ed1,ed2,ed3,ed4;

	STRFormatPutSafe(dummy,FONTSIZE_MAXLEN,"%g",pt2double(g->look->bodyfontsize));

	catch( PWCreate(win,&box,PW_TYPE_DIRECTION,NULL) );
	catch( PWCreate(box,NULL,PW_TYPE_LABEL,
						PW_SCALE_FACTOR, 0,
						PW_LABEL_TEXT, LabelFontSize,
						PW_LABEL_UNDERLINE, 's',
						NULL) );
	catch( PWCreate(box,&ed1,PW_TYPE_EDLINE,
						PW_EDLINE_MAXLENGTH, FONTSIZE_MAXLEN,
						PW_EDLINE_SET, dummy,
						PW_EDLINE_ACTION_AFTER, fontsizesel,
						PW_EDLINE_KEYPRESS, 's',
						NULL) );

	catch( PWCreate(win,&box,PW_TYPE_DIRECTION, NULL));
	catch( PWCreate(box,NULL,PW_TYPE_LABEL,
						PW_LABEL_TEXT, LabelDevice,
						PW_LABEL_UNDERLINE, 'e',
						PW_SCALE_FACTOR, 0,
						NULL) );
	catch( PWCreate(box,&g->device,PW_TYPE_EDLINE,
						PW_EDLINE_MAXLENGTH, IO_MAXFULLNAME,
						PW_EDLINE_SET, g->devicename,
						PW_EDLINE_KEYPRESS, 'e',
						PW_EDLINE_ACTION_AFTER, devicesel,
						NULL) );
	ed2=g->device;

	catch( PWCreate(win,&box,PW_TYPE_DIRECTION, NULL));
	catch( PWCreate(box,NULL,PW_TYPE_LABEL,
						PW_LABEL_TEXT, LabelDriver,
						PW_LABEL_UNDERLINE, 'r',
						PW_SCALE_FACTOR, 0,
						NULL) );
	catch( PWCreate(box,&g->driver,PW_TYPE_LOOSE_ITEM,
						PW_LOOSE_TEXT, g->drivername,
						PW_LOOSE_KEYPRESS, 'r',
						PW_LOOSE_CHANGE_STATUS, FALSE,
						PW_LOOSE_AUTOSIZE, FALSE,
						PW_LOOSE_ACTION_HIT, driversel,
						NULL) );

	catch( PWCreate(win,NULL,PW_TYPE_LOOSE_ITEM,
						PW_LOOSE_TEXT, ItemPrintSub,
						PW_LOOSE_ACTION_HIT,hitprtsub,
						PW_LOOSE_WINDOW_DO, TRUE,
						PW_LOOSE_KEYPRESS, 'i',
						PW_LOOSE_STATUS, g->printsub ? PW_STATUS_SELECTED : PW_STATUS_AVAILABLE,
						NULL) );
	catch( PWCreate(win,NULL,PW_TYPE_LOOSE_ITEM,
						PW_LOOSE_TEXT, ItemPrintMargin,
						PW_LOOSE_ACTION_HIT,hitprtmarg,
						PW_LOOSE_WINDOW_DO, TRUE,
						PW_LOOSE_KEYPRESS, 'm',
						PW_LOOSE_STATUS, g->printmarg ? PW_STATUS_SELECTED : PW_STATUS_AVAILABLE,
						NULL) );

	catch(PWCreate(win,&box,PW_TYPE_DIRECTION, NULL));
	catch(PWCreate(box,NULL,PW_TYPE_LABEL,
						PW_LABEL_TEXT, LabelFrom,
						PW_LABEL_UNDERLINE, 'f',
						PW_SCALE_FACTOR, 0,
						NULL));
	STRFormatPut(dummy,"%d",g->printfirst);
	catch(PWCreate(box,&ed3,PW_TYPE_EDLINE,
						PW_EDLINE_MAXLENGTH, 10,
						PW_EDLINE_SET, dummy,
						PW_EDLINE_KEYPRESS, 'f',
						PW_EDLINE_ACTION_AFTER, intsel,
						PW_OBJECT_AUXILIARY, &g->printfirst,
						PW_EDLINE_ACTION_TEST, PW_EDLINE_ACTION_TEST_NATURAL,
						PW_EDLINE_WIDTH_FS, short2pt(3),
						NULL));
	catch(PWCreate(box,NULL,PW_TYPE_LABEL,
						PW_LABEL_TEXT, LabelTo,
						PW_LABEL_UNDERLINE, 't',
						PW_SCALE_FACTOR, 0,
						NULL));
	STRFormatPut(dummy,"%d",g->printlast);
	catch(PWCreate(box,&ed4,PW_TYPE_EDLINE,
						PW_EDLINE_MAXLENGTH, 10,
						PW_EDLINE_SET, dummy,
						PW_EDLINE_KEYPRESS, 't',
						PW_EDLINE_ACTION_AFTER, intsel,
						PW_OBJECT_AUXILIARY, &g->printlast,
						PW_EDLINE_ACTION_TEST, PW_EDLINE_ACTION_TEST_NATURAL,
						PW_EDLINE_WIDTH_FS, short2pt(3),
						NULL));

	PWChange(ed1,PW_EDLINE_EDLINE_NEXT,ed2,
				 PW_EDLINE_EDLINE_PREV,ed4,NULL);
	PWChange(ed2,PW_EDLINE_EDLINE_NEXT,ed3,
				 PW_EDLINE_EDLINE_PREV,ed1,NULL);
	PWChange(ed3,PW_EDLINE_EDLINE_NEXT,ed4,
				 PW_EDLINE_EDLINE_PREV,ed2,NULL);
	PWChange(ed4,PW_EDLINE_EDLINE_NEXT,ed1,
				 PW_EDLINE_EDLINE_PREV,ed3,NULL);

	err=PWActivate(win);
	return err;
}

static Error doprint(PWObject object)
{
	Global *g;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);
	g->loaded=TRUE;
	return PWChange(object,PW_SYSTEM_BREAKDOWN,NULL);
}

static Error print(PWObject object)
{
	Error err=ERR_OK;
	Global *g;
	PWObject win;
	pt fontsize;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);
	if (!g->text) return err;
	fontsize=g->look->bodyfontsize;

	err=PWCreate(NULL,&win,PW_TYPE_OUTLINE,
					PW_OUTLINE_QUIT,
					PW_OUTLINE_QUIT_KEYPRESS, 27,
					PW_OUTLINE_TITLE_TEXT, ItemPrint,
					PW_GLOBAL_AUXILIARY, g,
					PW_WINDOW_NOYSCALE,
					PW_OUTLINE_ACTION_DO, doprint,
					NULL);
	if (!err)
	{
		printmenuhandle(g,win);
		PWRemove(win);
		if (g->loaded)
		{
			g->printing=TRUE;
			FileDisplay(g,"");
			g->printing=FALSE;
		}
		g->look->bodyfontsize=fontsize;
		if (g->loaded)
		{
			reload(object);
			g->loaded=FALSE;
		}
	}
	return ERR_OK;
}

static Error bookmark(PWObject object)
{
	Error err;
	Global *g;
	PWObject win=NULL,menu,box;

	catch( PWQuery(object,PW_GLOBAL_AUXILIARY,&g) );

	err=PWCreate(NULL,&win,PW_TYPE_OUTLINE,
					PW_OUTLINE_QUIT,
					PW_OUTLINE_QUIT_KEYPRESS, 27,
					PW_OUTLINE_TITLE_TEXT, ItemBookmarks,
					PW_GLOBAL_AUXILIARY, g,
					NULL);
	if (!err) err=PWCreate(win,&box,PW_TYPE_DIRECTION,
					PW_SCALE_FACTOR, 0,
					NULL);
	if (!err) err=PWCreate(box,NULL,PW_TYPE_LOOSE_ITEM,
					PW_LOOSE_TEXT, ItemBookmarkAdd,
					PW_LOOSE_KEYPRESS, 'a',
					PW_LOOSE_ACTION_HIT, BookmarkAdd,
					PW_LOOSE_CHANGE_STATUS, FALSE,
					NULL);
	if (!err) err=PWCreate(box,NULL,PW_TYPE_LOOSE_ITEM,
					PW_LOOSE_TEXT, ItemBookmarkLoad,
					PW_LOOSE_KEYPRESS, 'l',
					PW_LOOSE_ACTION_HIT, BookmarkLoad,
					PW_LOOSE_CHANGE_STATUS, FALSE,
					NULL);
	if (!err) err=PWCreate(box,NULL,PW_TYPE_LOOSE_ITEM,
					PW_LOOSE_TEXT, ItemBookmarkSave,
					PW_LOOSE_KEYPRESS, 's',
					PW_LOOSE_ACTION_HIT, BookmarkSave,
					PW_LOOSE_CHANGE_STATUS, FALSE,
					NULL);
	if (!err) err=PWCreate(win,NULL,PW_TYPE_SEPARATOR,NULL);
	if (!err) err=PWCreate(win,&menu,PW_TYPE_MENU,
					PW_MENU_ACTION_SELECT, BookmarkSelect,
					PW_MENU_VISIBLE_LINES, 4,
					NULL);
	g->menu=menu;
	if (!err) err=BookmarkMenu(g,menu);
	if (!err) err=PWActivate(win);
	if (win) PWRemove(win);
	if (g->loaded)
	{
		FileDisplay(g, "");
		if (g->text) PWChange(g->title,PW_INFOSTRING_TEXT,g->text->title,NULL);
		g->loaded=FALSE;
	}
	return ERR_OK;
}
